# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-11-10 12:22
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models
import eoxserver.resources.coverages.models
import re


class Migration(migrations.Migration):

    dependencies = [
        ('coverages', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='browsetype',
            name='alpha_nodata_value',
            field=models.FloatField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='browsetype',
            name='alpha_range_max',
            field=models.FloatField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='browsetype',
            name='alpha_range_min',
            field=models.FloatField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='browsetype',
            name='blue_nodata_value',
            field=models.FloatField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='browsetype',
            name='blue_range_max',
            field=models.FloatField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='browsetype',
            name='blue_range_min',
            field=models.FloatField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='browsetype',
            name='green_nodata_value',
            field=models.FloatField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='browsetype',
            name='green_range_max',
            field=models.FloatField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='browsetype',
            name='green_range_min',
            field=models.FloatField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='browsetype',
            name='red_or_grey_nodata_value',
            field=models.FloatField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='browsetype',
            name='red_or_grey_range_max',
            field=models.FloatField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='browsetype',
            name='red_or_grey_range_min',
            field=models.FloatField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='browsetype',
            name='alpha_expression',
            field=models.CharField(blank=True, max_length=512, null=True, validators=[eoxserver.resources.coverages.models.band_expression_validator]),
        ),
        migrations.AlterField(
            model_name='browsetype',
            name='blue_expression',
            field=models.CharField(blank=True, max_length=512, null=True, validators=[eoxserver.resources.coverages.models.band_expression_validator]),
        ),
        migrations.AlterField(
            model_name='browsetype',
            name='green_expression',
            field=models.CharField(blank=True, max_length=512, null=True, validators=[eoxserver.resources.coverages.models.band_expression_validator]),
        ),
        migrations.AlterField(
            model_name='browsetype',
            name='name',
            field=models.CharField(blank=True, max_length=256, validators=[django.core.validators.RegexValidator(re.compile(b'^[a-zA-z_][a-zA-Z0-9_]*$'), message=b'This field must contain a valid Name.')]),
        ),
        migrations.AlterField(
            model_name='browsetype',
            name='red_or_grey_expression',
            field=models.CharField(blank=True, max_length=512, null=True, validators=[eoxserver.resources.coverages.models.band_expression_validator]),
        ),
    ]
