<div class="navbar navbar-inverse navbar-fixed-top">
  <div class="container">
    <div class="collapse navbar-collapse">
      <ul class="nav navbar-nav" id="tab-headers-main">
        <li><a class="tab-header" target="_blank" href="{{url}}" style="font-size:30px"><strong>{{title}}</strong></a></li>
        <li class="divider-vertical"></li>
      </ul>
    </div><!--/.nav-collapse -->
  </div>
</div>
