# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-06-04 14:26
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('coverages', '0005_eo_object_identifier_validator'),
        ('services', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='servicevisibility',
            name='eo_object',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='service_visibility', to='coverages.EOObject'),
        ),
        migrations.AlterField(
            model_name='servicevisibility',
            name='service',
            field=models.CharField(choices=[(b'wms', b'WMS'), (b'wcs', b'WCS'), (b'os', b'OpenSearch'), (b'wc', b'WebClient')], max_length=4),
        ),
        migrations.AlterUniqueTogether(
            name='servicevisibility',
            unique_together=set([('eo_object', 'service')]),
        ),
    ]
