(function(){"use strict";var a=this;a.require(["app","backbone","communicator","backbone.marionette"],function(a,b,c){a.addInitializer(function(c){a.router=new d,b.history.start({pushState:!1})});var d=b.Marionette.AppRouter.extend({initialize:function(a){this.listenTo(c.mediator,"router:setUrl",this.setUrl)},setUrl:function(b){b.x=Math.round(100*b.x)/100,b.y=Math.round(100*b.y)/100;var c="map/"+b.x+"/"+b.y+"/"+b.l;a.router.navigate(c,{trigger:!1})},routes:{"map/:x/:y/:l":"centerAndZoom"},centerAndZoom:function(a,b,d){c.mediator.trigger("map:center",{x:a,y:b,l:d})}});return d})}).call(this);