<div class="input-group">
    <span class="input-group-addon">
        <input type="radio" name="selections" value="{{id}}"/>
    </span>
    <span class="form-control">{{name}} &ndash; {{date}}</span>
    <span class="input-group-btn">
    	<button class="btn btn-danger delete-selection" type="button">
    		<i class="fa fa-minus-circle"></i>
    	</button>
    </span>
</div>