#-------------------------------------------------------------------------------
#
# Project: EOxServer <http://eoxserver.org>
# Authors: Fabian Schindler <fabian.schindler@eox.at>
#
#-------------------------------------------------------------------------------
# Copyright (C) 2013 EOX IT Services GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies of this Software or works derived from this Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#-------------------------------------------------------------------------------


from eoxserver.services.parameters import (
    VersionedParams, CapabilitiesRenderParams
)


class WCSParamsMixIn(object):
    def __iter__(self):
        for k, v in super(WCSParamsMixIn, self).__iter__():
            yield k, v
        
        yield ("service", "WCS")


class WCSCapabilitiesRenderParams(WCSParamsMixIn, CapabilitiesRenderParams):
    pass


class CoverageRenderParams(WCSParamsMixIn, VersionedParams):
    def __init__(self, coverage, version):
        super(CoverageRenderParams, self).__init__(version)
        self._coverage = coverage


    @property
    def coverage(self):
        return self._coverage


    @property
    def coverage_id(self):
        return self.coverage.identifier


    coverage_id_key_name = None

    def __iter__(self):
        for k, v in super(CoverageRenderParams, self).__iter__():
            yield k, v

        yield ("request", "GetCoverage")
        yield (self.coverage_id_key_name, self.coverage_id)


class CoverageDescriptionRenderParams(WCSParamsMixIn, VersionedParams):
    def __init__(self, coverages, version):
        super(CoverageDescriptionRenderParams, self).__init__(version)
        self._coverages = coverages

    @property
    def coverages(self):
        return self._coverages

    @property
    def coverage_ids(self):
        return [coverage.identifier for coverage in self.coverages]


    coverage_ids_key_name = None

    def __iter__(self):
        for k, v in super(CoverageDescriptionRenderParams, self).__iter__():
            yield k, v

        yield ("request", "DescribeCoverage")
        yield (self.coverage_ids_key_name, ",".join(self.coverage_ids))

