(function(){"use strict";var a=this;a.define(["backbone","communicator","underscore"],function(a,b,c){var d=a.Marionette.CollectionView.extend({tagName:"ul",className:"radio",initialize:function(a){},onShow:function(a){}});return{BaseLayerSelectionView:d}})}).call(this);