from .getcapabilities import WCS20GetCapabilitiesHandler
from .describecoverage import WCS20DescribeCoverageHandler
from .getcoverage import WCS20GetCoverageHandler
from .describeeocoverageset import WCS20DescribeEOCoverageSetHandler
from .geteocoverageset import WCS20GetEOCoverageSetHandler


GetCapabilitiesHandler = WCS20GetCapabilitiesHandler
DescribeCoverageHandler = WCS20DescribeCoverageHandler
DescribeEOCoverageSetHandler = WCS20DescribeEOCoverageSetHandler
GetEOCoverageSetHandler = WCS20GetEOCoverageSetHandler
GetCoverageHandler = WCS20GetCoverageHandler
