# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-11-13 15:38
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('coverages', '0002_browse_type_fields'),
    ]

    operations = [
        migrations.AddField(
            model_name='metadataitem',
            name='semantic',
            field=models.SmallIntegerField(blank=True, choices=[(0, b'other'), (1, b'description'), (2, b'documentation'), (3, b'thumbnail')], null=True),
        ),
        migrations.AlterUniqueTogether(
            name='metadataitem',
            unique_together=set([('eo_object', 'semantic')]),
        ),
    ]
