# -*- coding: utf-8 -*-
# Generated by Django 1.11.26 on 2019-11-13 14:23
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('coverages', '0005_eo_object_identifier_validator'),
    ]

    operations = [
        migrations.AddField(
            model_name='masktype',
            name='validity',
            field=models.BooleanField(default=False),
        ),
    ]
