<div class="panel-heading">
    <button type="button" class="close"><i class="fa fa-times-circle"></i></button>
    <h3 class="panel-title"><i class="fa fa-fw fa-pencil-square"></i> Selection Manager</h3>
</div>
<div class="panel-body" id="selection-list"></div>
<div class="panel-footer">
  <div id="fileupload" >
    <div class="myfileupload-buttonbar ">
      <label class="btn btn-default myui-button">
        <span><i class="fa fa-fw fa-upload"></i> Upload Selection</span>
        <input id="upload-selection" type="file" name="files[]" />
      </label>
      <button type="button" class="btn btn-default" id="btn-export-selection"><i class="fa fa-fw fa-download"></i> Export Selection</button>
      <button type="button" class="btn btn-default" id="btn-save-selection"><i class="fa fa-fw fa-save"></i> Save Selection</button>
  </div>
</div>
