# -*- coding: utf-8 -*-
# Generated by Django 1.11.23 on 2020-02-06 15:13
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('backends', '0002_storage_auth'),
    ]

    operations = [
        migrations.AlterField(
            model_name='storage',
            name='name',
            field=models.CharField(blank=True, max_length=1024, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name='storageauth',
            name='name',
            field=models.CharField(blank=True, max_length=1024, null=True, unique=True),
        ),
    ]
