{% load static %}
<!doctype html>

<!--
 Project: EOxClient <https://github.com/EOX-A/EOxClient>
 Authors: Daniel Santillan <daniel.santillan@eox.at>

 Copyright (C) 2014 EOX IT Services GmbH

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all
 copies of this Software or works derived from this Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
-->

<!--[if lt IE 7]>      <html class="no-js lt-ie9 lt-ie8 lt-ie7"> <![endif]-->
<!--[if IE 7]>         <html class="no-js lt-ie9 lt-ie8"> <![endif]-->
<!--[if IE 8]>         <html class="no-js lt-ie9"> <![endif]-->
<!--[if gt IE 8]><!--> <html class="no-js"> <!--<![endif]-->
<head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">
    <title>EOxServer - Webclient</title>
    <meta name="description" content="">
    <meta name="viewport" content="width=device-width">

    <link rel="icon" href="{% static 'favicon.ico' %}" type="image/x-icon">
    <link rel="shortcut icon" href="% static '}favicon.ico' %}" type="image/x-icon">
    <link rel="stylesheet" href="{% static 'styles/main.css' %}">
    <script src="{% static 'scripts/vendor/modernizr.js' %}"></script>
</head>
<body>

    <!--[if lt IE 7]>
        <p class="chromeframe">You are using an <strong>outdated</strong> browser. Please <a href="http://browsehappy.com/">upgrade your browser</a> or <a href="http://www.google.com/chromeframe/?redirect=true">activate Google Chrome Frame</a> to improve your experience.</p>
    <![endif]-->

    <script data-main="{% static 'scripts/init' %}" src="{% static 'bower_components/requirejs/require.js' %}"></script>

    <div id="content">
        <div id="map" class="not-selectable"></div>
        <div id="viewContent"></div>
        <div id="leftSideBar"></div>
        <div id="rightSideBar"></div>
        <div id="topBar"></div>
        <div id="bottomBar"></div>
    </div>

    <div id="loadscreen" class="modal-backdrop"> <i class="fa fa-cog fa-spin fa-4x"></i> </div>

    <!-- HIDDEN DOWNLOAD DIV -->
    <div id="div-downloads">
      <iframe name="iframe-download-post" id="iframe-download-post"></iframe>
    </div>


    <div id="preload">
       <img src="{% static 'images/glyphicons.png' %}" width="1" height="1" alt="Preload image">
    </div>

    <div id="error-messages"></div>


     <div id="terrain_attribution">
        <p style="float: right"><a href="javascript:;" onclick="toggle(terrain_attribution)">×</a></p>
        <p>Terrain map data</p>
        <ul>
            <li>OpenStreetMap © <a href="http://www.openstreetmap.org/copyright" target="_blank">OpenStreetMap contributors</a></li>
            <li>NaturalEarth <a href="http://www.naturalearthdata.com/about/terms-of-use/" target="_blank">public domain</a></li>
            <li>EU-DEM © Produced using Copernicus data and information<br>funded by the European Union</li>
            <li>SRTM © <a href="http://nasa.gov" target="_blank">NASA</a></li>
            <li>GTOPO30</li>
            <li>CleanTOPO2 public domain</li>
            <li>GlobCover © <a href="http://esa.int" target="_blank">ESA</a></li>
        </ul>
        <p>Terrain map design © <a href="http://eox.at" target="_blank">EOX IT Services GmbH</a></p>
    </div>

    <script>
        // Used to display/hide terrain attribution
        function toggle(e) { e.style.display = (e.style.display == 'block' ? 'none' : 'block'); }
    </script>



    <!-- Google Analytics: change UA-XXXXX-X to be your site's ID. -->
    <!--<script>
        var _gaq=[['_setAccount','UA-XXXXX-X'],['_trackPageview']];
        (function(d,t){var g=d.createElement(t),s=d.getElementsByTagName(t)[0];
        g.src='//www.google-analytics.com/ga.js';
        s.parentNode.insertBefore(g,s)}(document,'script'));
    </script>-->
</body>
</html>
