# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-08-28 10:02
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('coverages', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Extra',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('display_name', models.CharField(blank=True, max_length=64, null=True)),
                ('info', models.TextField(blank=True, null=True)),
                ('color', models.CharField(blank=True, max_length=64, null=True)),
                ('default_visible', models.BooleanField(default=False)),
                ('eo_object', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='webclient_extra', to='coverages.EOObject')),
            ],
        ),
    ]
