(function(){"use strict";var a=this;a.require.config({deps:["backbone.marionette","bootstrap","marionette.handlebars","main"],shim:{handlebars:{exports:"Handlebars"},backbone:{deps:["underscore","jquery"],exports:"Backbone"},bootstrap:{deps:["jquery"],exports:"jquery"},libcoverage:{deps:["backbone"]},filesaver:{exports:"saveAs"},lm:{exports:"lm"},timeslider:{deps:["d3"]},timeslider_plugins:{deps:["timeslider","libcoverage"]},openlayers:{exports:"ol"}},paths:{requirejs:"../bower_components/requirejs/require",jquery:"../bower_components/jquery/jquery.min",jqueryui:"../bower_components/jquery-ui/ui/minified/jquery-ui.min",backbone:"../bower_components/backbone-amd/backbone-min",underscore:"../bower_components/underscore-amd/underscore-min",d3:"../bower_components/d3/d3.min",timeslider:"../bower_components/d3.TimeSlider/d3.timeslider.min",timeslider_plugins:"../bower_components/d3.TimeSlider/d3.timeslider.plugins.min",libcoverage:"../bower_components/libcoverage/libcoverage.min",filesaver:"../bower_components/filesaver/FileSaver",lm:"../bower_components/lm.js/lm","backbone.marionette":"../bower_components/backbone.marionette/lib/core/amd/backbone.marionette.min","backbone.wreqr":"../bower_components/backbone.wreqr/lib/amd/backbone.wreqr.min","backbone.babysitter":"../bower_components/backbone.babysitter/lib/backbone.babysitter.min",bootstrap:"../bower_components/bootstrap/dist/js/bootstrap.min",text:"../bower_components/requirejs-text/text",tmpl:"../templates",handlebars:"../bower_components/require-handlebars-plugin/Handlebars",i18nprecompile:"../bower_components/require-handlebars-plugin/hbs/i18nprecompile",json2:"../bower_components/require-handlebars-plugin/hbs/json2",hbs:"../bower_components/require-handlebars-plugin/hbs","marionette.handlebars":"../bower_components/backbone.marionette.handlebars/backbone.marionette.handlebars.min",openlayers:"../bower_components/ol3/ol"},hbs:{disableI18n:!0}})}).call(this);