(function(){"use strict";var a=this;a.define(["backbone"],function(a){var b=a.Model.extend({name:"",timeSlider:!1,timeSliderProtocol:"",color:"",time:null,visible:null,opacity:0,view:{id:"",protocol:"",urls:[],style:"default",isBaseLayer:null,attribution:"",matrixSet:"",format:"",resolutions:[],maxExtent:[],projection:"",gutter:null,buffer:null,units:"",transitionEffect:"",isphericalMercator:null,wrapDateLine:null,zoomOffset:null,requestEncoding:"KVP"},download:{id:"",protocol:"",url:[]}});return{LayerModel:b}})}).call(this);