<!--
#-------------------------------------------------------------------------------
#
# Project: EOxServer <http://eoxserver.org>
# Authors: Stephan Krause <stephan.krause@eox.at>
#          Stephan Meissl <stephan.meissl@eox.at>
#
#-------------------------------------------------------------------------------
# Copyright (C) 2012 EOX IT Services GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies of this Software or works derived from this Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#-------------------------------------------------------------------------------
-->
{% load static %}
<html>
<head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">
    <title>EOxServer</title>
    <meta name="description" content="">
    <meta name="viewport" content="width=device-width">

    <link rel="icon" href="{% static 'favicon.ico' %}" type="image/x-icon">
    <link rel="shortcut icon" href="{% static 'favicon.ico' %}" type="image/x-icon">
</head>
<body>
    

    <h1>Welcome to this EOxServer instance</h1>

    <h2>The following components are available:</h2>
    <ul>
        <li><a href="{% url 'ows' %}">OGC Web Services base URL</a></li>
            <ul>
                <li><a href="{% url 'ows' %}?service=wms&amp;request=getcapabilities">EO-WMS</a></li>
                <li><a href="{% url 'ows' %}?service=wcs&amp;request=getcapabilities">EO-WCS</a></li>
                <li><a href="{% url 'ows' %}?service=wps&amp;request=getcapabilities">WPS</a></li>
            </ul>
        <li><a href="{% url 'opensearch:description' %}">OpenSearch</a></li>
        <li><a href="{% url 'webclient:index' %}">Web Client</a></li>
        
        <li><a href="{% url 'admin:index' %}">Admin Client</a></li>
    </ul>

    Powered by<br/>
    <a href="http://eoxserver.org" target="_blank"><img src="{% static 'EOxServer_logo_small.png' %}" alt="EOxServer"/></a><br/>
    version {{version}}<br/>
    See <a href="https://docs.eoxserver.org" target="_blank">EOxServer documentation</a> for help.
</body>
</html>
