<div class="input-group">
    <span class="input-group-addon">
        <input type="checkbox" {{#if visible}}checked="checked"{{/if}}/>
    </span>
    <span class="form-control">{{name}}</span>
    <span class="input-group-addon">
        <i class="fa fa-sort"></i>
        <i class="fa fa-square" style="color: {{color}}"></i>
        <i class="fa fa-adjust" data-toggle="popover" data-content="n"></i>
    </span>
</div>
