<div class="panel-heading">
    <button type="button" class="close"><i class="fa fa-times-circle"></i></button>
    <i class="fa fa-wrench no-label"></i>
</div>
<div class="panel-body">
    <div id="selection"></div>
    <div id="visualization"></div>
</div>
