# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-06-04 14:26
from __future__ import unicode_literals

from django.db import migrations, models
import eoxserver.resources.coverages.models


class Migration(migrations.Migration):

    dependencies = [
        ('coverages', '0004_auto_20181220_1300'),
    ]

    operations = [
        migrations.AlterField(
            model_name='eoobject',
            name='identifier',
            field=models.CharField(max_length=256, unique=True, validators=[eoxserver.resources.coverages.models.eo_object_identifier_validator]),
        ),
    ]
