<div class="panel-heading">
    <button type="button" class="close"><i class="fa fa-times-circle"></i></button>
    <h3 class="panel-title"><i class="fa fa-fw fa-globe"></i> Layers</h3>
</div>
<div class="panel-body">
    <div id="overlays"></div>
    <div id="products"></div>
    <div id="baseLayers"></div>
</div>
