(function(){"use strict";var a=this;a.define(["backbone","communicator","hbs!tmpl/UIElement","underscore"],function(a,b,c){var d=a.Marionette.ItemView.extend({template:{type:"handlebars",template:c},initialize:function(a){}});return{UIElementView:d}})}).call(this);