(function(){"use strict";var a=this;a.define(["backbone","backbone.marionette"],function(a){var b=a.Marionette.Controller.extend({initialize:function(b){this.mediator=new a.Wreqr.EventAggregator,this.mediator.on("all",function(a){"progress:change"!=a&&"router:setUrl"!=a&&console.log(a)}),this.reqres=new a.Wreqr.RequestResponse,this.command=new a.Wreqr.Commands,this.on("all")}});return new b})}).call(this);