<div class="modal-dialog">
  <div class="modal-content">
    <div class="modal-header">
      <button type="button" class="close" data-dismiss="modal"><i class="fa fa-times-circle"></i></button>
      <h3 class="panel-title" id="about-title"><i class="fa fa-fw fa-info-circle"></i> About EOxClient</h3>
    </div>
    <div class="modal-body modal-body-about">
      <p>EOxClient is a project to unify the needs of EO data visualization into a single configurable client.</p>
      <p>It is currently being developed as an open source project by <a href="http://eox.at">
      <i class="fa fa-external-link-square"></i> EOX IT Services GmbH</a>. We are a small, Vienna based company,
      specializing in earth observation data and software needed to access it.</p>
      <p>If you are interested in participating, check out our
      <a href="https://github.com/EOX-A/EOxClient"><i class="fa fa-github-square"></i> github repository</a> or
      drop us a note on <a href="https://twitter.com/eox_a"><i class="fa fa-twitter-square"></i> twitter</a>.</p>
    </div>
    <div class="modal-footer">
      <button type="button" class="btn btn-primary" data-dismiss="modal">Close</button>
    </div>
  </div>
</div>
